@@ -30,7 +30,6 @@ import java.util.*;
 import java.util.stream.Collectors;
 
 import scala.Option;
-import scala.collection.JavaConverters;
 import scala.collection.Seq;
 import scala.collection.mutable.Buffer;
 
@@ -81,6 +80,8 @@ import org.apache.comet.shims.ShimFileFormat;
 import org.apache.comet.vector.CometVector;
 import org.apache.comet.vector.NativeUtil;
 
+import static scala.jdk.javaapi.CollectionConverters.asJava;
+
 /**
  * A vectorized Parquet reader that reads a Parquet file in a batched fashion.
  *
@@ -255,7 +256,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
     ParquetReadOptions readOptions = builder.build();
 
     Map<String, String> objectStoreOptions =
-        JavaConverters.mapAsJavaMap(NativeConfig.extractObjectStoreOptions(conf, file.pathUri()));
+        asJava(NativeConfig.extractObjectStoreOptions(conf, file.pathUri()));
 
     // TODO: enable off-heap buffer when they are ready
     ReadOptions cometReadOptions = ReadOptions.builder(conf).build();
@@ -306,7 +307,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
       List<Type> fields = requestedSchema.getFields();
       List<Type> fileFields = fileSchema.getFields();
       ParquetColumn[] parquetFields =
-          JavaConverters.seqAsJavaList(parquetColumn.children()).toArray(new ParquetColumn[0]);
+          asJava(parquetColumn.children()).toArray(new ParquetColumn[0]);
       int numColumns = fields.size();
       if (partitionSchema != null) numColumns += partitionSchema.size();
       columnReaders = new AbstractColumnReader[numColumns];
@@ -409,6 +410,15 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
         }
       }
 
+      boolean encryptionEnabled = CometParquetUtils.encryptionEnabled(conf);
+
+      // Create keyUnwrapper if encryption is enabled
+      CometFileKeyUnwrapper keyUnwrapper = null;
+      if (encryptionEnabled) {
+        keyUnwrapper = new CometFileKeyUnwrapper();
+        keyUnwrapper.storeDecryptionKeyRetriever(file.filePath().toString(), conf);
+      }
+
       int batchSize =
           conf.getInt(
               CometConf.COMET_BATCH_SIZE().key(),
@@ -425,7 +435,8 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
               timeZoneId,
               batchSize,
               caseSensitive,
-              objectStoreOptions);
+              objectStoreOptions,
+              keyUnwrapper);
     }
     isInitialized = true;
   }
@@ -433,7 +444,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
   private ParquetColumn getParquetColumn(MessageType schema, StructType sparkSchema) {
     // We use a different config from the config that is passed in.
     // This follows the setting  used in Spark's SpecificParquetRecordReaderBase
-    Configuration config = new Configuration();
+    Configuration config = new Configuration(conf);
     config.setBoolean(SQLConf.PARQUET_BINARY_AS_STRING().key(), false);
     config.setBoolean(SQLConf.PARQUET_INT96_AS_TIMESTAMP().key(), false);
     config.setBoolean(SQLConf.CASE_SENSITIVE().key(), false);
@@ -618,14 +629,14 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
   }
 
   private void checkParquetType(ParquetColumn column) throws IOException {
-    String[] path = JavaConverters.seqAsJavaList(column.path()).toArray(new String[0]);
+    String[] path = asJava(column.path()).toArray(new String[0]);
     if (containsPath(fileSchema, path)) {
       if (column.isPrimitive()) {
         ColumnDescriptor desc = column.descriptor().get();
         ColumnDescriptor fd = fileSchema.getColumnDescription(desc.getPath());
         TypeUtil.checkParquetType(fd, column.sparkType());
       } else {
-        for (ParquetColumn childColumn : JavaConverters.seqAsJavaList(column.children())) {
+        for (ParquetColumn childColumn : asJava(column.children())) {
           checkColumn(childColumn);
         }
       }
@@ -645,7 +656,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
    * file schema, or whether it conforms to the type of the file schema.
    */
   private void checkColumn(ParquetColumn column) throws IOException {
-    String[] path = JavaConverters.seqAsJavaList(column.path()).toArray(new String[0]);
+    String[] path = asJava(column.path()).toArray(new String[0]);
     if (containsPath(fileSchema, path)) {
       if (column.isPrimitive()) {
         ColumnDescriptor desc = column.descriptor().get();
@@ -654,7 +665,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
           throw new UnsupportedOperationException("Schema evolution not supported.");
         }
       } else {
-        for (ParquetColumn childColumn : JavaConverters.seqAsJavaList(column.children())) {
+        for (ParquetColumn childColumn : asJava(column.children())) {
           checkColumn(childColumn);
         }
       }
@@ -805,7 +816,7 @@ public class NativeBatchReader extends RecordReader<Void, ColumnarBatch> impleme
   @SuppressWarnings("deprecation")
   private int loadNextBatch() throws Throwable {
 
-    for (ParquetColumn childColumn : JavaConverters.seqAsJavaList(parquetColumn.children())) {
+    for (ParquetColumn childColumn : asJava(parquetColumn.children())) {
       checkParquetType(childColumn);
     }
 
