@@ -24,14 +24,15 @@ import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.execution.datasources.PartitionedFile
 
 object ShimBatchReader {
+
   def newPartitionedFile(partitionValues: InternalRow, file: String): PartitionedFile =
     PartitionedFile(
       partitionValues,
-      SparkPath.fromUrlString(file),
+      SparkPath.fromPathString(file),
       -1, // -1 means we read the entire file
       -1,
       Array.empty[String],
       0,
-      0
-    )
+      0,
+      Map.empty)
 }
