@@ -15,6 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
+pub mod internal;
+pub mod monotonically_increasing_id;
 pub mod rand;
+pub mod randn;
 
 pub use rand::RandExpr;
+pub use randn::RandnExpr;
