@@ -21,10 +21,13 @@ package org.apache.spark.sql.comet
 
 import scala.reflect.ClassTag
 
+import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.SparkSession
 import org.apache.spark.sql.catalyst._
 import org.apache.spark.sql.catalyst.expressions._
+import org.apache.spark.sql.catalyst.plans.QueryPlan
 import org.apache.spark.sql.catalyst.plans.physical.{Partitioning, UnknownPartitioning}
+import org.apache.spark.sql.comet.shims.ShimStreamSourceAwareSparkPlan
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.datasources._
 import org.apache.spark.sql.execution.metric.{SQLMetric, SQLMetrics}
@@ -33,12 +36,12 @@ import org.apache.spark.util.collection._
 
 import com.google.common.base.Objects
 
-import org.apache.comet.DataTypeSupport
+import org.apache.comet.CometConf
 import org.apache.comet.parquet.CometParquetFileFormat
 import org.apache.comet.serde.OperatorOuterClass.Operator
 
 /**
- * Comet fully native scan node for DataSource V1.
+ * Comet fully native scan node for DataSource V1 that delegates to DataFusion's DataSourceExec.
  */
 case class CometNativeScanExec(
     override val nativeOp: Operator,
@@ -53,29 +56,51 @@ case class CometNativeScanExec(
     disableBucketedScan: Boolean = false,
     originalPlan: FileSourceScanExec,
     override val serializedPlanOpt: SerializedPlan)
-    extends CometLeafExec {
+    extends CometLeafExec
+    with DataSourceScanExec
+    with ShimStreamSourceAwareSparkPlan {
 
-  override def nodeName: String =
-    s"${super.nodeName}: ${tableIdentifier.map(_.toString).getOrElse("")}"
+  override lazy val metadata: Map[String, String] = originalPlan.metadata
 
-  override def outputPartitioning: Partitioning =
+  override val nodeName: String =
+    s"CometNativeScan $relation ${tableIdentifier.map(_.unquotedString).getOrElse("")}"
+
+  override lazy val outputPartitioning: Partitioning =
     UnknownPartitioning(originalPlan.inputRDD.getNumPartitions)
 
-  override def outputOrdering: Seq[SortOrder] = originalPlan.outputOrdering
+  override lazy val outputOrdering: Seq[SortOrder] = originalPlan.outputOrdering
+
+  override def doCanonicalize(): CometNativeScanExec = {
+    CometNativeScanExec(
+      nativeOp,
+      relation,
+      output.map(QueryPlan.normalizeExpressions(_, output)),
+      requiredSchema,
+      QueryPlan.normalizePredicates(
+        CometScanUtils.filterUnusedDynamicPruningExpressions(partitionFilters),
+        output),
+      optionalBucketSet,
+      optionalNumCoalescedBuckets,
+      QueryPlan.normalizePredicates(dataFilters, output),
+      None,
+      disableBucketedScan,
+      originalPlan.doCanonicalize(),
+      SerializedPlan(None))
+  }
 
   override def stringArgs: Iterator[Any] = Iterator(output)
 
   override def equals(obj: Any): Boolean = {
     obj match {
       case other: CometNativeScanExec =>
-        this.output == other.output &&
+        this.originalPlan == other.originalPlan &&
         this.serializedPlanOpt == other.serializedPlanOpt
       case _ =>
         false
     }
   }
 
-  override def hashCode(): Int = Objects.hashCode(output)
+  override def hashCode(): Int = Objects.hashCode(originalPlan, serializedPlanOpt)
 
   override lazy val metrics: Map[String, SQLMetric] = {
     // We don't append CometMetricNode.baselineMetrics because
@@ -153,9 +178,14 @@ case class CometNativeScanExec(
           sparkContext,
           "Time spent reading and parsing metadata from the footer"))
   }
+
+  /**
+   * See [[org.apache.spark.sql.execution.DataSourceScanExec.inputRDDs]]. Only used for tests.
+   */
+  override def inputRDDs(): Seq[RDD[InternalRow]] = originalPlan.inputRDDs()
 }
 
-object CometNativeScanExec extends DataTypeSupport {
+object CometNativeScanExec {
   def apply(
       nativeOp: Operator,
       scanExec: FileSourceScanExec,
@@ -177,12 +207,13 @@ object CometNativeScanExec extends DataTypeSupport {
     // https://github.com/apache/arrow-datafusion-comet/issues/190
     def transform(arg: Any): AnyRef = arg match {
       case _: HadoopFsRelation =>
-        scanExec.relation.copy(fileFormat = new CometParquetFileFormat)(session)
+        scanExec.relation.copy(fileFormat =
+          new CometParquetFileFormat(CometConf.SCAN_NATIVE_DATAFUSION))(session)
       case other: AnyRef => other
       case null => null
     }
 
-    val newArgs = mapProductIterator(scanExec, transform(_))
+    val newArgs = mapProductIterator(scanExec, transform)
     val wrapped = scanExec.makeCopy(newArgs).asInstanceOf[FileSourceScanExec]
     val batchScanExec = CometNativeScanExec(
       nativeOp,
@@ -200,12 +231,4 @@ object CometNativeScanExec extends DataTypeSupport {
     scanExec.logicalLink.foreach(batchScanExec.setLogicalLink)
     batchScanExec
   }
-
-  override def isAdditionallySupported(dt: DataType): Boolean = {
-    // TODO add array and map
-    dt match {
-      case s: StructType => s.fields.map(_.dataType).forall(isTypeSupported)
-      case _ => false
-    }
-  }
 }
