@@ -29,9 +29,9 @@ import org.apache.spark.sql.execution.{SparkPlan, TakeOrderedAndProjectExec, Una
 import org.apache.spark.sql.execution.metric.{SQLMetric, SQLMetrics, SQLShuffleReadMetricsReporter, SQLShuffleWriteMetricsReporter}
 import org.apache.spark.sql.vectorized.ColumnarBatch
 
+import org.apache.comet.CometSparkSessionExtensions.withInfo
 import org.apache.comet.serde.QueryPlanSerde.exprToProto
 import org.apache.comet.serde.QueryPlanSerde.supportedSortType
-import org.apache.comet.shims.ShimCometTakeOrderedAndProjectExec
 
 /**
  * Comet physical plan node for Spark `TakeOrderedAndProjectExec`.
@@ -44,6 +44,7 @@ case class CometTakeOrderedAndProjectExec(
     override val originalPlan: SparkPlan,
     override val output: Seq[Attribute],
     limit: Int,
+    offset: Int,
     sortOrder: Seq[SortOrder],
     projectList: Seq[NamedExpression],
     child: SparkPlan)
@@ -57,7 +58,8 @@ case class CometTakeOrderedAndProjectExec(
     "dataSize" -> SQLMetrics.createSizeMetric(sparkContext, "data size"),
     "numPartitions" -> SQLMetrics.createMetric(
       sparkContext,
-      "number of partitions")) ++ readMetrics ++ writeMetrics
+      "number of partitions")) ++ readMetrics ++ writeMetrics ++ CometMetricNode.shuffleMetrics(
+    sparkContext)
 
   private lazy val serializer: Serializer =
     new UnsafeRowSerializer(child.output.size, longMetric("dataSize"))
@@ -68,7 +70,7 @@ case class CometTakeOrderedAndProjectExec(
 
   protected override def doExecuteColumnar(): RDD[ColumnarBatch] = {
     val childRDD = child.executeColumnar()
-    if (childRDD.getNumPartitions == 0) {
+    if (childRDD.getNumPartitions == 0 || limit == 0) {
       new ParallelCollectionRDD(sparkContext, Seq.empty[ColumnarBatch], 1, Map.empty)
     } else {
       val singlePartitionRDD = if (childRDD.getNumPartitions == 1) {
@@ -77,12 +79,13 @@ case class CometTakeOrderedAndProjectExec(
         val localTopK = if (orderingSatisfies) {
           CometExecUtils.getNativeLimitRDD(childRDD, child.output, limit)
         } else {
-          childRDD.mapPartitionsInternal { iter =>
+          val numParts = childRDD.getNumPartitions
+          childRDD.mapPartitionsWithIndexInternal { case (idx, iter) =>
             val topK =
               CometExecUtils
                 .getTopKNativePlan(child.output, sortOrder, child, limit)
                 .get
-            CometExec.getCometIterator(Seq(iter), child.output.length, topK)
+            CometExec.getCometIterator(Seq(iter), child.output.length, topK, numParts, idx)
           }
         }
 
@@ -100,9 +103,9 @@ case class CometTakeOrderedAndProjectExec(
 
       singlePartitionRDD.mapPartitionsInternal { iter =>
         val topKAndProjection = CometExecUtils
-          .getProjectionNativePlan(projectList, child.output, sortOrder, child, limit)
+          .getProjectionNativePlan(projectList, child.output, sortOrder, child, limit, offset)
           .get
-        val it = CometExec.getCometIterator(Seq(iter), output.length, topKAndProjection)
+        val it = CometExec.getCometIterator(Seq(iter), output.length, topKAndProjection, 1, 0)
         setSubqueries(it.id, this)
 
         Option(TaskContext.get()).foreach { context =>
@@ -121,19 +124,31 @@ case class CometTakeOrderedAndProjectExec(
     val orderByString = truncatedString(sortOrder, "[", ",", "]", maxFields)
     val outputString = truncatedString(output, "[", ",", "]", maxFields)
 
-    s"CometTakeOrderedAndProjectExec(limit=$limit, orderBy=$orderByString, output=$outputString)"
+    s"CometTakeOrderedAndProjectExec(limit=$limit, offset=$offset, " +
+      s"orderBy=$orderByString, output=$outputString)"
   }
 
   override protected def withNewChildInternal(newChild: SparkPlan): SparkPlan =
     this.copy(child = newChild)
 }
 
-object CometTakeOrderedAndProjectExec extends ShimCometTakeOrderedAndProjectExec {
-  // TODO: support offset for Spark 3.4
+object CometTakeOrderedAndProjectExec {
   def isSupported(plan: TakeOrderedAndProjectExec): Boolean = {
-    val exprs = plan.projectList.map(exprToProto(_, plan.child.output))
-    val sortOrders = plan.sortOrder.map(exprToProto(_, plan.child.output))
-    exprs.forall(_.isDefined) && sortOrders.forall(_.isDefined) && getOffset(plan).getOrElse(
-      0) == 0 && supportedSortType(plan, plan.sortOrder)
+    val exprs = plan.projectList.map { p =>
+      val o = exprToProto(p, plan.child.output)
+      if (o.isEmpty) {
+        withInfo(plan, s"unsupported projection: $p")
+      }
+      o
+    }
+    val sortOrders = plan.sortOrder.map { s =>
+      val o = exprToProto(s, plan.child.output)
+      if (o.isEmpty) {
+        withInfo(plan, s"unsupported sort order: $s")
+      }
+      o
+    }
+    exprs.forall(_.isDefined) && sortOrders.forall(_.isDefined) &&
+    supportedSortType(plan, plan.sortOrder)
   }
 }
