@@ -77,6 +77,7 @@ abstract class CometTestBase
     conf.set(SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key, "1g")
     conf.set(SQLConf.ADAPTIVE_AUTO_BROADCASTJOIN_THRESHOLD.key, "1g")
     conf.set(CometConf.COMET_ENABLED.key, "true")
+    conf.set(CometConf.COMET_ENABLE_ONHEAP_MODE.key, "true")
     conf.set(CometConf.COMET_EXEC_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_SHUFFLE_ENABLED.key, "true")
     conf.set(CometConf.COMET_RESPECT_PARQUET_FILTER_PUSHDOWN.key, "true")
@@ -88,6 +89,15 @@ abstract class CometTestBase
     conf
   }
 
+  protected def isFeatureEnabled(feature: String): Boolean = {
+    try {
+      NativeBase.isFeatureEnabled(feature)
+    } catch {
+      case _: Throwable =>
+        false
+    }
+  }
+
   /**
    * A helper function for comparing Comet DataFrame with Spark result using absolute tolerance.
    */
@@ -204,7 +214,7 @@ abstract class CometTestBase
           assert(
             false,
             s"Expected only Comet native operators, but found ${op.nodeName}.\n" +
-              s"plan: $plan")
+              s"plan: ${new ExtendedExplainInfo().generateVerboseExtendedInfo(plan)}")
         }
     }
   }
