@@ -421,6 +421,76 @@ object CometShuffleBenchmark extends CometBenchmarkBase {
     }
   }
 
+  def shuffleRangePartitionBenchmark(
+      values: Int,
+      dataType: DataType,
+      width: Int,
+      partitionNum: Int): Unit = {
+    val benchmark =
+      new Benchmark(
+        s"SQL Wide ($width cols) ${dataType.sql} Range Partition Shuffle($partitionNum Partition)",
+        values,
+        output = output)
+
+    val projection = (1 to width)
+      .map(i => s"CAST(CAST(RAND(1) * 100 AS INTEGER) AS ${dataType.sql}) AS c$i")
+      .mkString(", ")
+    val columns = (1 to width).map(i => s"c$i").mkString(", ")
+
+    withTempPath { dir =>
+      withTempTable("parquetV1Table") {
+        prepareTable(dir, spark.sql(s"SELECT $projection FROM $tbl"))
+
+        benchmark.addCase("SQL Parquet - Spark") { _ =>
+          spark
+            .sql(s"select $columns from parquetV1Table")
+            .repartitionByRange(partitionNum, Column("c1"))
+            .noop()
+        }
+
+        benchmark.addCase("SQL Parquet - Comet (Spark Shuffle)") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "false") {
+            spark
+              .sql(s"select $columns from parquetV1Table")
+              .repartitionByRange(partitionNum, Column("c1"))
+              .noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet (Comet Arrow Shuffle)") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
+            CometConf.COMET_SHUFFLE_MODE.key -> "jvm") {
+            spark
+              .sql(s"select $columns from parquetV1Table")
+              .repartitionByRange(partitionNum, Column("c1"))
+              .noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet (Comet Shuffle)") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_ENABLED.key -> "true",
+            CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
+            CometConf.COMET_SHUFFLE_MODE.key -> "native") {
+            spark
+              .sql(s"select $columns from parquetV1Table")
+              .repartitionByRange(partitionNum, Column("c1"))
+              .noop()
+          }
+        }
+
+        benchmark.run()
+      }
+    }
+  }
+
   override def runCometBenchmark(mainArgs: Array[String]): Unit = {
     runBenchmarkWithTable("Shuffle on array", 1024 * 1024 * 1) { v =>
       Seq(
@@ -591,5 +661,69 @@ object CometShuffleBenchmark extends CometBenchmarkBase {
           shuffleWideBenchmark(v, dataType, 20, 201)
         }
     }
+
+    runBenchmarkWithTable("Wide Range Partition Shuffle (10 cols)", 1024 * 1024 * 10) { v =>
+      Seq(
+        BooleanType,
+        ByteType,
+        ShortType,
+        IntegerType,
+        LongType,
+        FloatType,
+        DoubleType,
+        StringType,
+        DecimalType(10, 0))
+        .foreach { dataType =>
+          shuffleRangePartitionBenchmark(v, dataType, 10, 5)
+        }
+    }
+
+    runBenchmarkWithTable("Wide Range Partition Shuffle (20 cols)", 1024 * 1024 * 10) { v =>
+      Seq(
+        BooleanType,
+        ByteType,
+        ShortType,
+        IntegerType,
+        LongType,
+        FloatType,
+        DoubleType,
+        StringType,
+        DecimalType(10, 0))
+        .foreach { dataType =>
+          shuffleRangePartitionBenchmark(v, dataType, 20, 5)
+        }
+    }
+
+    runBenchmarkWithTable("Wide Range Partition Shuffle (10 cols)", 1024 * 1024 * 10) { v =>
+      Seq(
+        BooleanType,
+        ByteType,
+        ShortType,
+        IntegerType,
+        LongType,
+        FloatType,
+        DoubleType,
+        StringType,
+        DecimalType(10, 0))
+        .foreach { dataType =>
+          shuffleRangePartitionBenchmark(v, dataType, 10, 201)
+        }
+    }
+
+    runBenchmarkWithTable("Wide Range Partition Shuffle (20 cols)", 1024 * 1024 * 10) { v =>
+      Seq(
+        BooleanType,
+        ByteType,
+        ShortType,
+        IntegerType,
+        LongType,
+        FloatType,
+        DoubleType,
+        StringType,
+        DecimalType(10, 0))
+        .foreach { dataType =>
+          shuffleRangePartitionBenchmark(v, dataType, 20, 201)
+        }
+    }
   }
 }
