@@ -62,6 +62,9 @@ import org.apache.comet.CometSparkSessionExtensions.{isSpark35Plus, isSpark40Plu
  * }}}
  */
 trait CometPlanStabilitySuite extends DisableAdaptiveExecutionSuite with TPCDSBase {
+  protected val scanImpls: Seq[String] =
+    Seq(CometConf.SCAN_AUTO, CometConf.SCAN_NATIVE_ICEBERG_COMPAT)
+
   protected val baseResourcePath: File = {
     getWorkspaceFilePath("spark", "src", "test", "resources", "tpcds-plan-stability").toFile
   }
@@ -89,79 +92,64 @@ trait CometPlanStabilitySuite extends DisableAdaptiveExecutionSuite with TPCDSBa
     new File(goldenFilePath, goldenFileName)
   }
 
-  private def isApproved(
-      dir: File,
-      actualSimplifiedPlan: String,
-      actualExplain: String): Boolean = {
-    val simplifiedFile = new File(dir, "simplified.txt")
-    val expectedSimplified = FileUtils.readFileToString(simplifiedFile, StandardCharsets.UTF_8)
-    val explainFile = new File(dir, "explain.txt")
-    val expectedExplain = FileUtils.readFileToString(explainFile, StandardCharsets.UTF_8)
-    expectedSimplified == actualSimplifiedPlan && expectedExplain == actualExplain
-  }
-
   /**
    * Serialize and save this SparkPlan. The resulting file is used by [[checkWithApproved]] to
    * check stability.
    *
-   * @param plan
-   *   the SparkPlan
-   * @param name
-   *   the name of the query
+   * @param dir
+   *   the directory to write to
+   * @param simplified
+   *   the simplified plan
    * @param explain
    *   the full explain output; this is saved to help debug later as the simplified plan is not
    *   too useful for debugging
    */
-  private def generateGoldenFile(plan: SparkPlan, name: String, explain: String): Unit = {
-    val dir = getDirForTest(name)
-    val simplified = getSimplifiedPlan(plan)
-    val foundMatch = dir.exists() && isApproved(dir, simplified, explain)
-
-    if (!foundMatch) {
-      FileUtils.deleteDirectory(dir)
-      if (!dir.mkdirs()) {
-        fail(s"Could not create dir: $dir")
-      }
-
-      val file = new File(dir, "simplified.txt")
-      FileUtils.writeStringToFile(file, simplified, StandardCharsets.UTF_8)
-      val fileOriginalPlan = new File(dir, "explain.txt")
-      FileUtils.writeStringToFile(fileOriginalPlan, explain, StandardCharsets.UTF_8)
-      logDebug(s"APPROVED: $file $fileOriginalPlan")
+  private def generateGoldenFile(dir: File, simplified: String, explain: String): Unit = {
+    FileUtils.deleteDirectory(dir)
+    if (!dir.mkdirs()) {
+      fail(s"Could not create dir: $dir")
     }
+
+    val file = new File(dir, "simplified.txt")
+    FileUtils.writeStringToFile(file, simplified, StandardCharsets.UTF_8)
+    val fileOriginalPlan = new File(dir, "explain.txt")
+    FileUtils.writeStringToFile(fileOriginalPlan, explain, StandardCharsets.UTF_8)
+    logDebug(s"APPROVED: $file $fileOriginalPlan")
   }
 
-  private def checkWithApproved(plan: SparkPlan, name: String, explain: String): Unit = {
-    val dir = getDirForTest(name)
+  private def checkWithApproved(
+      dir: File,
+      name: String,
+      simplified: String,
+      explain: String): Unit = {
+
+    val approvedSimplifiedFile = new File(dir, "simplified.txt")
+    val approvedExplainFile = new File(dir, "explain.txt")
+
+    // write actual files out for debugging
     val tempDir = FileUtils.getTempDirectory
-    val actualSimplified = getSimplifiedPlan(plan)
-    val foundMatch = isApproved(dir, actualSimplified, explain)
+    val actualSimplifiedFile = new File(tempDir, s"$name.actual.simplified.txt")
+    val actualExplainFile = new File(tempDir, s"$name.actual.explain.txt")
+    FileUtils.writeStringToFile(actualSimplifiedFile, simplified, StandardCharsets.UTF_8)
+    FileUtils.writeStringToFile(actualExplainFile, explain, StandardCharsets.UTF_8)
 
-    if (!foundMatch) {
-      // show diff with last approved
-      val approvedSimplifiedFile = new File(dir, "simplified.txt")
-      val approvedExplainFile = new File(dir, "explain.txt")
-
-      val actualSimplifiedFile = new File(tempDir, s"$name.actual.simplified.txt")
-      val actualExplainFile = new File(tempDir, s"$name.actual.explain.txt")
-
-      val approvedSimplified =
-        FileUtils.readFileToString(approvedSimplifiedFile, StandardCharsets.UTF_8)
-      // write out for debugging
-      FileUtils.writeStringToFile(actualSimplifiedFile, actualSimplified, StandardCharsets.UTF_8)
-      FileUtils.writeStringToFile(actualExplainFile, explain, StandardCharsets.UTF_8)
+    comparePlans("simplified", approvedSimplifiedFile, actualSimplifiedFile)
+    comparePlans("explain", approvedExplainFile, actualExplainFile)
+  }
 
+  private def comparePlans(planType: String, expectedFile: File, actualFile: File): Unit = {
+    val expected = FileUtils.readFileToString(expectedFile, StandardCharsets.UTF_8)
+    val actual = FileUtils.readFileToString(actualFile, StandardCharsets.UTF_8)
+    if (expected != actual) {
       fail(s"""
-           |Plans did not match:
-           |last approved simplified plan: ${approvedSimplifiedFile.getAbsolutePath}
-           |last approved explain plan: ${approvedExplainFile.getAbsolutePath}
-           |
-           |$approvedSimplified
-           |
-           |actual simplified plan: ${actualSimplifiedFile.getAbsolutePath}
-           |actual explain plan: ${actualExplainFile.getAbsolutePath}
-           |
-           |$actualSimplified
+              |Plans did not match:
+              |last approved $planType plan: ${expectedFile.getAbsolutePath}
+              |
+              |$expected
+              |
+              |actual $planType plan: ${actualFile.getAbsolutePath}
+              |
+              |$actual
         """.stripMargin)
     }
   }
@@ -274,13 +262,16 @@ trait CometPlanStabilitySuite extends DisableAdaptiveExecutionSuite with TPCDSBa
       val qe = sql(queryString).queryExecution
       val plan = qe.executedPlan
       val explain = normalizeLocation(normalizeIds(qe.explainString(FormattedMode)))
-
+      val simplified = getSimplifiedPlan(plan)
       assert(ValidateRequirements.validate(plan))
 
+      val name = query + suffix
+      val dir = getDirForTest(name)
+
       if (regenerateGoldenFiles) {
-        generateGoldenFile(plan, query + suffix, explain)
+        generateGoldenFile(dir, simplified, explain)
       } else {
-        checkWithApproved(plan, query + suffix, explain)
+        checkWithApproved(dir, name, simplified, explain)
       }
     }
   }
@@ -295,8 +286,6 @@ trait CometPlanStabilitySuite extends DisableAdaptiveExecutionSuite with TPCDSBa
     conf.set(MEMORY_OFFHEAP_SIZE.key, "2g")
     conf.set(CometConf.COMET_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_ENABLED.key, "true")
-    conf.set(CometConf.COMET_NATIVE_SCAN_ENABLED.key, "true")
-    conf.set(CometConf.COMET_NATIVE_SCAN_IMPL.key, CometConf.SCAN_AUTO)
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "1g")
     conf.set(CometConf.COMET_EXEC_SHUFFLE_ENABLED.key, "true")
 
@@ -315,10 +304,12 @@ class CometTPCDSV1_4_PlanStabilitySuite extends CometPlanStabilitySuite {
   override val goldenFilePath: String =
     new File(baseResourcePath, planName).getAbsolutePath
 
-  if (CometConf.COMET_NATIVE_SCAN_IMPL.get() == CometConf.SCAN_AUTO) {
+  scanImpls.foreach { scan =>
     tpcdsQueries.foreach { q =>
-      test(s"check simplified (tpcds-v1.4/$q)") {
-        testQuery("tpcds", q)
+      test(s"check simplified (tpcds-v1.4/$q) - $scan") {
+        withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> scan) {
+          testQuery("tpcds", q)
+        }
       }
     }
   }
@@ -335,10 +326,12 @@ class CometTPCDSV2_7_PlanStabilitySuite extends CometPlanStabilitySuite {
   override val goldenFilePath: String =
     new File(baseResourcePath, planName).getAbsolutePath
 
-  if (CometConf.COMET_NATIVE_SCAN_IMPL.get() == CometConf.SCAN_AUTO) {
+  scanImpls.foreach { scan =>
     tpcdsQueriesV2_7_0.foreach { q =>
-      test(s"check simplified (tpcds-v2.7.0/$q)") {
-        testQuery("tpcds-v2.7.0", q)
+      test(s"check simplified (tpcds-v2.7.0/$q) - $scan") {
+        withSQLConf(CometConf.COMET_NATIVE_SCAN_IMPL.key -> scan) {
+          testQuery("tpcds-v2.7.0", q)
+        }
       }
     }
   }
