@@ -21,5 +21,4 @@ package org.apache.comet.shims
 
 import org.apache.spark.sql.SQLQueryTestHelper
 
-trait ShimCometTPCHQuerySuite extends SQLQueryTestHelper {
-}
+trait ShimCometTPCHQuerySuite extends SQLQueryTestHelper {}
